//
//  AsyncViewController.m
//  PAGViewer
//
//  Created by zhangquan06 on 2024/7/29.
//  Copyright © 2024 libpag.org. All rights reserved.
//

#import "AsyncViewController.h"
#import <libpag/PAGView.h>
#import <libpag/PAGDiskCache.h>
#import <libpag/PAGFile.h>

@interface AsyncViewController () <PAGViewListener>

@property (nonatomic, strong) PAGView *pagView;

@property (nonatomic, copy) NSString *pathURLStr;

@end

@implementation AsyncViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    
    // 移除缓存以模拟首次调用
    [PAGDiskCache RemoveAll];
    
    self.pagView = [[PAGView alloc] initWithFrame:CGRectMake(0.0f, 0.0f, 300, 300)];
    [self.view addSubview:self.pagView];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    // 复现路径A
    [self testPAGFileLoadAsync];
    
    // 复现路径B
    [self testPAGViewSetPathAsync];
}

// 路径A：使用PAGFile loadAsync接口
- (void)testPAGFileLoadAsync {
    // path对象在test方法执行完了会释放
    // 当请求完成时，PAG内部[PAGDiskCacheImpl WritFile:path data:cacheData] 执行崩溃
    NSString *path = [NSString stringWithFormat:@"%@", @"https://pag.art/file/like.pag"];
    [PAGFile LoadAsync:path completionBlock:^(PAGFile *file) {
        
    }];
}

// 路径B：多次调用 PAGView setPathAsync
- (void)testPAGViewSetPathAsync {
    self.pathURLStr = [NSString stringWithFormat:@"%@", @"https://pag.art/file/like.pag"];
    [self.pagView setPathAsync:self.pathURLStr completionBlock:^(PAGFile *file) {
        
    }];
    
    // 模拟网速慢时，旧的文件还在下载，播放新的URL
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.2 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        // 旧的字符串释放
        // 传入新的path字符串对象
        self.pathURLStr = [NSString stringWithFormat:@"%@", @"https://pag.art/file/like.pag"];
        [self.pagView setPathAsync:self.pathURLStr completionBlock:^(PAGFile *file) {
            
        }];
    });
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
